package java.nio;

import sun.misc.Cleaner;
import sun.misc.Unsafe;
import sun.nio.ch.DirectBuffer;

class DirectFloatBufferU extends FloatBuffer implements DirectBuffer
{
    protected static final Unsafe unsafe;
    private static final long arrayBaseOffset;
    protected static final boolean unaligned;
    private final Object att;
    
    @Override
    public Object attachment() {
        return this.att;
    }
    
    public Cleaner cleaner() {
        return null;
    }
    
    DirectFloatBufferU(DirectBuffer att, int n, int n2, int n3, int n4, int n5) {
        super(n, n2, n3, n4);
        this.address = att.address() + (long)n5;
        this.att = att;
    }
    
    @Override
    public FloatBuffer slice() {
        int position = this.position();
        int limit = this.limit();
        int n = (position <= limit) ? (limit - position) : 0;
        int n2 = position << 2;
        assert n2 >= 0;
        return new DirectFloatBufferU(this, -1, 0, n, n, n2);
    }
    
    @Override
    public FloatBuffer duplicate() {
        return new DirectFloatBufferU(this, this.markValue(), this.position(), this.limit(), this.capacity(), 0);
    }
    
    @Override
    public FloatBuffer asReadOnlyBuffer() {
        return new DirectFloatBufferRU(this, this.markValue(), this.position(), this.limit(), this.capacity(), 0);
    }
    
    @Override
    public long address() {
        return this.address;
    }
    
    private long ix(int n) {
        return this.address + ((long)n << 2);
    }
    
    @Override
    public float get() {
        return DirectFloatBufferU.unsafe.getFloat(this.ix(this.nextGetIndex()));
    }
    
    @Override
    public float get(int n) {
        return DirectFloatBufferU.unsafe.getFloat(this.ix(this.checkIndex(n)));
    }
    
    @Override
    public FloatBuffer get(float[] array, int n, int n2) {
        if ((long)n2 << 2 > 6L) {
            Buffer.checkBounds(n, n2, array.length);
            int position = this.position();
            int limit = this.limit();
            assert position <= limit;
            if (n2 > ((position <= limit) ? (limit - position) : 0)) {
                throw new BufferUnderflowException();
            }
            if (this.order() != ByteOrder.nativeOrder()) {
                Bits.copyToIntArray(this.ix(position), array, (long)n << 2, (long)n2 << 2);
            }
            else {
                Bits.copyToArray(this.ix(position), array, DirectFloatBufferU.arrayBaseOffset, (long)n << 2, (long)n2 << 2);
            }
            this.position(position + n2);
        }
        else {
            super.get(array, n, n2);
        }
        return this;
    }
    
    @Override
    public FloatBuffer put(float x) {
        DirectFloatBufferU.unsafe.putFloat(this.ix(this.nextPutIndex()), x);
        return this;
    }
    
    @Override
    public FloatBuffer put(int n, float x) {
        DirectFloatBufferU.unsafe.putFloat(this.ix(this.checkIndex(n)), x);
        return this;
    }
    
    @Override
    public FloatBuffer put(FloatBuffer floatBuffer) {
        if ((floatBuffer instanceof DirectFloatBufferU)) {
            if (floatBuffer == this) {
                throw new IllegalArgumentException();
            }
            DirectFloatBufferU directFloatBufferU = (DirectFloatBufferU)floatBuffer;
            int position2 = directFloatBufferU.position();
            int limit2 = directFloatBufferU.limit();
            assert position2 <= limit2;
            int n2 = (position2 <= limit2) ? (limit2 - position2) : 0;
            int position3 = this.position();
            int limit3 = this.limit();
            assert position3 <= limit3;
            if (n2 > ((position3 <= limit3) ? (limit3 - position3) : 0)) {
                throw new BufferOverflowException();
            }
            DirectFloatBufferU.unsafe.copyMemory(directFloatBufferU.ix(position2), this.ix(position3), (long)n2 << 2);
            directFloatBufferU.position(position2 + n2);
            this.position(position3 + n2);
        }
        else if (floatBuffer.hb != null) {
            int position = floatBuffer.position();
            int limit = floatBuffer.limit();
            assert position <= limit;
            int n = (position <= limit) ? (limit - position) : 0;
            this.put(floatBuffer.hb, floatBuffer.offset + position, n);
            floatBuffer.position(position + n);
        }
        else {
            super.put(floatBuffer);
        }
        return this;
    }
    
    @Override
    public FloatBuffer put(float[] array, int n, int n2) {
        if ((long)n2 << 2 > 6L) {
            Buffer.checkBounds(n, n2, array.length);
            int position = this.position();
            int limit = this.limit();
            assert position <= limit;
            if (n2 > ((position <= limit) ? (limit - position) : 0)) {
                throw new BufferOverflowException();
            }
            if (this.order() != ByteOrder.nativeOrder()) {
                Bits.copyFromIntArray(array, (long)n << 2, this.ix(position), (long)n2 << 2);
            }
            else {
                Bits.copyFromArray(array, DirectFloatBufferU.arrayBaseOffset, (long)n << 2, this.ix(position), (long)n2 << 2);
            }
            this.position(position + n2);
        }
        else {
            super.put(array, n, n2);
        }
        return this;
    }
    
    @Override
    public FloatBuffer compact() {
        int position = this.position();
        int limit = this.limit();
        assert position <= limit;
        int n = (position <= limit) ? (limit - position) : 0;
        DirectFloatBufferU.unsafe.copyMemory(this.ix(position), this.ix(0), (long)n << 2);
        this.position(n);
        this.limit(this.capacity());
        this.discardMark();
        return this;
    }
    
    @Override
    public boolean isDirect() {
        return true;
    }
    
    @Override
    public boolean isReadOnly() {
        return false;
    }
    
    @Override
    public ByteOrder order() {
        return (ByteOrder.nativeOrder() != ByteOrder.BIG_ENDIAN) ? ByteOrder.LITTLE_ENDIAN : ByteOrder.BIG_ENDIAN;
    }
    
    static {
        unsafe = Bits.unsafe();
        arrayBaseOffset = (long)DirectFloatBufferU.unsafe.arrayBaseOffset(float[].class);
        unaligned = Bits.unaligned();
    }
}
